/************************************************************
* Copyright (C) 2007-2008 Masahiko SAWAI All Rights Reserved. 
************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <wiiremote.h>
#include <wiiremote_utils.h>

static int done;

static 
void
signal_handler(int i)
{
	fprintf(stderr, "sigint_handler()\n");
	done = 1;
}

int main(int argc, char *argv[])
{
	int number, i;
	WRMT_WiiRemote *wiiremotes[WRMT_MAX_DEVICES];
	
	if (WRMT_Init() != 0)
	{
		fprintf(stderr, "WRMT_Init() Faild.\n");
		exit(-1);
	}
	atexit(WRMT_Quit);

	number = WRMT_GetNumWiiRemote();
	if (number <= 0)
	{
		fprintf(stderr, "No Wii Remote Found.\n");
		exit(-1);
	}
	fprintf(stderr, "Number of Wii Remote : %d\n", number);

	/* Open and init all WiiRemotes */
	for (i = 0;i < number;i++)
	{
		WRMT_IOReturn rc;

		wiiremotes[i] = WRMT_GetWiiRemoteAt(i);
		rc = WRMT_WiiRemote_Open(wiiremotes[i]);
		if (rc == WRMT_IO_ERROR)
		{
			fprintf(stderr, "WRMT_WiiRemote_Open(%d) Fail.\n", i);
			fprintf(stderr, "\t%s\n", WRMT_GetError());
			exit(-1);
		}
		fprintf(stderr, "[%d] WRMT_WiiRemote_Open() Success.\n", i);
	}

	/* Main Loop */
	signal(SIGINT, signal_handler);
	done = 0;
	while (!done)
	{
		int updated_index;
		while ((updated_index = WRMT_Poll()) >= 0)
		{
			int buttons, leds;
			/* Get device status */
			buttons = WRMT_WiiRemote_GetState(wiiremotes[updated_index], WRMT_DATA_BUTTONS);

			/* LEDs control */
			if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_PLUS|WRMT_MASK_BUTTON_MINUS))
			{
				leds = WRMT_WiiRemote_GetState(wiiremotes[updated_index], WRMT_DATA_LEDS);
				if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_PLUS)) leds++;
				if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_MINUS)) leds--;

				if (leds < 0) leds = 0;
				else if (leds > 0x0f) leds = 0x0f;

				/* Set LEDs status */
				WRMT_WiiRemote_SetState(wiiremotes[updated_index], WRMT_DATA_LEDS, leds);
				WRMT_WiiRemote_Update(wiiremotes[updated_index]);
			}

			/* Exit if HOME is pressed. */
			if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_HOME))
			{
				done = 1;
				break;
			}
		}
		
		fflush(stderr);
		WRMT_Sleep(30);
	}

	/* Close All WiiRemotes */
	fprintf(stderr, "\n");
	for (i = WRMT_GetNumWiiRemote()-1;i >= 0;i--)
	{
		fprintf(stderr, "WRMT_WiiRemote_Close(%d).\n", i);
		WRMT_WiiRemote_Close(wiiremotes[i]);
	}
	fprintf(stderr, "WRMT_Quit().\n");

	return 0;
}
