/************************************************************
* Copyright (C) 2007-2008 Masahiko SAWAI All Rights Reserved. 
************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <wiiremote.h>
#include <wiiremote_utils.h>

static int done;

static 
void
signal_handler(int i)
{
	done = 1;
}

int main(int argc, char *argv[])
{
	int number, i;
	WRMT_WiiRemote *wiiremotes[WRMT_MAX_DEVICES];
	
	if (WRMT_Init() != 0)
	{
		fprintf(stderr, "WRMT_Init() Faild.\n");
		exit(-1);
	}
	atexit(WRMT_Quit);

	/* Get Number of WiiRemotes */
	number = WRMT_GetNumWiiRemote();
	if (number <= 0)
	{
		fprintf(stderr, "No Wii Remote Found.\n");
		exit(-1);
	}
	fprintf(stderr, "Number of Wii Remote : %d\n", number);

	/* Open and init all WiiRemotes */
	for (i = 0;i < number;i++)
	{
		WRMT_IOReturn rc;

		wiiremotes[i] = WRMT_GetWiiRemoteAt(i);
		rc = WRMT_WiiRemote_Open(wiiremotes[i]);
		if (rc == WRMT_IO_ERROR)
		{
			fprintf(stderr, "WRMT_WiiRemote_Open(%d) Fail.\n", i);
			fprintf(stderr, "\t%s\n", WRMT_GetError());
			exit(-1);
		}
		fprintf(stderr, "[%d] WRMT_WiiRemote_Open() Success.\n", i);

		WRMT_WiiRemote_SetEnabled(wiiremotes[i], WRMT_FUNCTION_MOTION, 1);
		WRMT_WiiRemote_Update(wiiremotes[i]);
	}

	/* Main Loop */
	signal(SIGINT, signal_handler);
	done = 0;
	while (!done)
	{
		int updated_index;
		while ((updated_index = WRMT_Poll()) >= 0)
		{
			WRMT_WiiRemote *wiiremote = wiiremotes[updated_index];

			fprintf(stderr, "[%d] : x:%d, y:%d, z:%d\n", updated_index,
				WRMT_WiiRemote_GetState(wiiremote, WRMT_DATA_MOTION_X),
				WRMT_WiiRemote_GetState(wiiremote, WRMT_DATA_MOTION_Y),
				WRMT_WiiRemote_GetState(wiiremote, WRMT_DATA_MOTION_Z));

			if (WRMT_WiiRemote_CheckButton(wiiremote, WRMT_MASK_BUTTON_HOME))
			{
				done = 1;
				break;
			}
		}

		WRMT_Sleep(30);
	}

	/* Close All WiiRemote */
	fprintf(stderr, "\n");
	for (i = WRMT_GetNumWiiRemote()-1;i >= 0;i--)
	{
		fprintf(stderr, "WRMT_WiiRemote_Close(%d).\n", i);
		WRMT_WiiRemote_Close(wiiremotes[i]);
	}
	fprintf(stderr, "WRMT_Quit().\n");

	return 0;
}
