/************************************************************
* Copyright (C) 2008 Masahiko SAWAI All Rights Reserved. 
************************************************************/
/**
* RIGHT/LEFT : sampling rate up/down
* UP/DOWN    : volume up/down
**/

#include <stdio.h>
#include <stdlib.h>

#include <wiiremote.h>
#include <wiiremote_utils.h>

static int done;

int main(int argc, char *argv[])
{
	int number, i;
	WRMT_WiiRemote *wiiremotes[WRMT_MAX_DEVICES];
	size_t sound_data_index = 0;
	unsigned char sound_data[] = {
		0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a,
		0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a,
		0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a,
		0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a, 0x7a,
	};
	size_t sound_data_size = sizeof(sound_data)/sizeof(unsigned char);
	
	if (WRMT_Init() != 0)
	{
		fprintf(stderr, "WRMT_Init() Faild.\n");
		exit(-1);
	}
	atexit(WRMT_Quit);

	number = WRMT_GetNumWiiRemote();
	fprintf(stderr, "Number of Wii Remote : %d\n", number);
	if (number <= 0)
	{
		fprintf(stderr, "No Wii Remote Found.\n");
		exit(-1);
	}

	for (i = 0;i < number;i++)
	{
		WRMT_IOReturn rc;

		wiiremotes[i] = WRMT_GetWiiRemoteAt(i);
		rc = WRMT_WiiRemote_Open(wiiremotes[i]);
		if (rc == WRMT_IO_ERROR)
		{
			fprintf(stderr, "WRMT_WiiRemote_Open(%d) Fail.\n", i);
			fprintf(stderr, "\t%s\n", WRMT_GetError());
			exit(-1);
		}
		fprintf(stderr, "[%d] WRMT_WiiRemote_Open() Success.\n", i);

		WRMT_WiiRemote_SetEnabled(wiiremotes[i], WRMT_FUNCTION_SPEAKER, 1);
		WRMT_WiiRemote_SetState(wiiremotes[i], WRMT_DATA_SPEAKER_FORMAT, 0x00); /* 0100 0000 */
		WRMT_WiiRemote_SetState(wiiremotes[i], WRMT_DATA_SPEAKER_VOLUME, 0x04);
		WRMT_WiiRemote_SetState(wiiremotes[i], WRMT_DATA_SPEAKER_SAMPLE_RATE, 0x11);
		WRMT_WiiRemote_Update(wiiremotes[i]);
	}


	done = 0;
	while (!done)
	{
		WRMT_UpdateAll();

		for (i = 0;i < number;i++)
		{
			int buttons = WRMT_WiiRemote_GetState(wiiremotes[i], WRMT_DATA_BUTTONS);

			/* change volume */
			if (WRMT_CheckButtonAny(buttons, WRMT_MASK_BUTTON_UP|WRMT_MASK_BUTTON_DOWN))
			{
				int volume = WRMT_WiiRemote_GetState(wiiremotes[i], WRMT_DATA_SPEAKER_VOLUME);
				if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_UP))
				{
					volume += 1;
					if (volume > 0xff) volume = 0xff;
				}
				else if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_DOWN))
				{
					volume -= 1;
					if (volume < 0) volume = 0;
				}
				fprintf(stderr, "[%d] volume : %d\n", i, volume);
				WRMT_WiiRemote_SetState(wiiremotes[i], WRMT_DATA_SPEAKER_VOLUME, volume);
			}

			/* change sample rate */
			if (WRMT_CheckButtonAny(buttons, WRMT_MASK_BUTTON_LEFT|WRMT_MASK_BUTTON_RIGHT))
			{
				int rate = WRMT_WiiRemote_GetState(wiiremotes[i], WRMT_DATA_SPEAKER_SAMPLE_RATE);
				if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_RIGHT))
				{
					rate += 1;
					if (rate > 0xff) rate = 0xff;
				}
				else if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_LEFT))
				{
					rate -= 1;
					if (rate < 0) rate = 0;
				}
				fprintf(stderr, "[%d] rate : %d\n", i, rate);
				
				WRMT_WiiRemote_SetState(wiiremotes[i], WRMT_DATA_SPEAKER_SAMPLE_RATE, rate);
			}

			/* play sound */
			if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_A))
			{
				size_t size = 20;
				if (sound_data_index + 20 > sound_data_size)
					size = sound_data_index + 20 - sound_data_size;

				WRMT_WiiRemote_PlaySound(wiiremotes[i], sound_data + sound_data_index, size); 
				sound_data_index += size;
				if (sound_data_index >= sound_data_size)
					sound_data_index = 0;
			}

			if (WRMT_CheckButton(buttons, WRMT_MASK_BUTTON_HOME)) done = 1;
		}
		fflush(stderr);
		WRMT_Sleep(10);
	}

	fprintf(stderr, "WRMT_Quit().\n");
	exit(0);
	return 0;
}
